/*{{{  externs.c*/
#include "externs.h"
#include "strmacs.h"
#include "symbol.h"
#include "unit.h"

/*{{{  global decls (incl. those in 'extern.h')*/
/*------------------------*/
/* Globals in 'externs.h' */
/*------------------------*/
Grouplist  *common_list,	/* lists of common blocks & namelist */
           *namelist_list;	/* groups with their contents.       */


/*----------------------------------*/
/* Globals 'private' to 'externs.c' */
/*----------------------------------*/
Symlist  *global_list;		/* list of *all* global entities used in
				   the program unit.  If a global 'name'
				   is used in several scope units, there
				   is a separate entity for each one. */

static char *str_func =		"an external function",
            *str_subr =		"an external subroutine",
            *str_module =	"a module",
            *str_program =	"the main program",
            *str_block_data =	"a block data unit",
            *str_common_block =	"a common block",
            *str_external =	"an external procedure";
            
#define  EXTERNAL_ENTITY_TYPE	(AT_FUNC | AT_SUBR | AT_MODULE | AT_PROGRAM |\
				 AT_BLOCK_DATA | AT_COMMON_BLOCK |\
				 AT_RESULT_VAR | AT_EXTERNAL)
/*}}}*/

/*--------------------------------------------------------*/
/* Functions for recording common block & namelist groups */
/*--------------------------------------------------------*/
/*{{{  Grouplist  *lookup_or_install_group (grplist_p, sp)*/
Grouplist  *lookup_or_install_group (grplist_p, sp)
/*-----------------------------------------------*/
	Grouplist  **grplist_p;	/* pointer to the head of a 'group list'
				   (viz 'common_list' or 'namelist_list') */
	Symbol  *sp;		/* group name -- NULL for blank common */
/*--------------------------------------------------------------*/
/*	Called by 'yyparse()' upon encountering a common or	*/
/* namelist statement, to lookup or install a 'group record'	*/
/* for the common block or namelist group -- in the 'group list'*/
/* starting at '*grplist_p' -- and return a pointer to it.	*/
/* 'sp' is the symtab entry for the group name (i.e. common	*/
/* block or namelist name).  It is NULL for a blank common.	*/
/*								*/
/* N.B. Because group records are added at the front of the	*/
/* appropriate 'group list' (i.e. 'common_list' or		*/
/* 'namelist_list'), the common blocks and namelist groups will	*/
/* be written out in the reverse order to that in the input	*/
/* program.							*/
/*--------------------------------------------------------------*/
{
	Grouplist  *gp;		/* pointer to a 'group record' */

	for (gp = *grplist_p;  gp;  gp = gp->next)
		if (gp->sp == sp)
			return  gp;	/* found it! */
	
	/*------------------------------------------------------*/
	/* First appearance of this 'group' in this scope unit.	*/
	/* Make a new record for it...				*/
	/*------------------------------------------------------*/
	gp = (Grouplist *) ecalloc (sizeof (Grouplist));
	gp->sp = sp;
	
	/*------------------------------------------------------*/
	/* ... and add it to the front of the 'group list'.	*/
	/*------------------------------------------------------*/
	gp->next = *grplist_p;
	*grplist_p = gp;

	return  gp;
}
/*}}}*/
/*{{{  install_group_member (gp, sp)*/
install_group_member (gp, sp)
/*-------------------------*/
	Grouplist  *gp;		/* pointer to a 'group record' */
	Symbol  *sp;		/* next object to install */
/*--------------------------------------------------------------*/
/*	Install 'sp' as the next object in the 'group'		*/
/* (i.e. common block or namelist) record  '*gp'.		*/
/*--------------------------------------------------------------*/
{
	Symlist  *idp;	/* object record */

	/*----------------------------------*/
	/* make an 'object record' for 'sp' */
	/*----------------------------------*/
	idp = (Symlist *) emalloc (sizeof (Symlist));
	idp->sp = sp;
	idp->next = NULL;

	/*------------------------------------------------------*/
	/* add it to the end of the object list in the 'group	*/
	/* record'.						*/
	/*------------------------------------------------------*/
	if (gp->last_id)
		gp->last_id = gp->last_id->next = idp;
	else 
		gp->first_id = gp->last_id = idp;

	return;
}
/*}}}*/


/*------------------------------------------------------*/
/* Function for maintaining list of global entity names	*/
/*------------------------------------------------------*/
/*{{{  bool  install_global_entity (sp, global_attr)*/
bool  install_global_entity (sp, global_attr)
/*-----------------------------------------*/
	Symbol *sp;
	int32  global_attr;	/* the 'global entity' flag to	*/
				/* install in 'sp->attrs'.	*/
/*{{{  DESCRIPTION*/
#if 0
-----
This is used to install in 'sp->attrs' an attribute, 'global_attr',
which signifies that 'sp' is a *global* rather than a local entity.

As all of the global entities in the program must have different
names, this function checks that 'sp->name' hasn't been used for a
different kind of global entity in the current program unit.
If everything is OK, it installs 'global_attr' in 'sp->attrs'
and returns TRUE, otherwise it gives an errmsg and returns FALSE.

Except in the case of (global_attr == AT_FUNC or AT_SUBR),
'global_attr' *shouldn't* be set in 'sp->attrs' prior to this call,
unless it was already set by a previous declaration.  This function
sets it if appropriate.  (This matters because the function doesn't
try to *unset* 'global_attr' if it's inappropriate.  It's also
important for the correctness of AT_COMMON_BLOCK -- see later).

'global_attr' is a flag from the group EXTERNAL_ENTITY_TYPE.
Their meanings in this context are as follows:

	flag		meaning when this function is invoked 
	----		-------------------------------------
	AT_PROGRAM	-- definition of program 'sp'       } must be OK, as
	AT_BLOCK_DATA	--      "     "  block data 'sp'    } it's the 1st ID
	AT_RESULT_VAR	--      "     "  external func 'sp' } in program unit

	AT_MODULE	--      "     or USE of module 'sp'    } must be OK if
	AT_SUBR		--      "     or interface of external } definition
			                (non dummy) subr 'sp'  } (see above)

	AT_FUNC		-- interface of external (non dummy) func 'sp'
	AT_COMMON_BLOCK	-- 'sp' is a common block name
	AT_EXTERNAL	-- 'sp' is given the EXTERNAL attribute


/*{{{  How it works*/
How it works
------------
The function keeps a list, 'global_list', of all the global
entities encountered so far.  If there is an entry with name 'sp->name'
in this list, an error is reported if its usage doesn't match that of
'global_attr'.   Some other tests are also performed, e.g.:

-- that a module does not USE itself;

-- that the EXTERNAL attribute or an explicit interface body is not
specified for the current program unit name (which is illegal even
though the program unit is a global entity).
(Incidentally, a related test, that an interface body isn't supplied
for a module or internal procedure, is performed by 'process_hdr_stmt()',
which performs various checks for procedure header stmts in interface
bodies before calling this!).

If these tests are passed, 'sp' is added to the front of the list if
it isn't already present.

Note that the same 'name' has different symtab entries for each scope
unit in which it appears.  E.g., in the following:

	subroutine s
	  common /c/ ...      ! scope 1
	  ...
	  contains
	    function f(x)
	      common /c/ ...  ! scope 2
	      ...

'c' has a different symtab entry for each of the two declarations
above, although it refers to the same global entity.  In this case,
each distinct symtab entry for 'name' (that refers to the global,
rather than a local, entity) is entered in the 'global_list'.
This is done, rather than just keeping one entry, because the different
entries might have different 'attr's set, even though they refer to
the same entity.  E.g. an external function 'E' may be referenced in
an EXTERNAL stmt in one scope unit and have an explicit interface in
another:  different symtab entries must be stored in order to record
exactly what is declared in each scope unit.
/*}}}*/

/*{{{  N.B. A tiny inefficiency (which can be ignored; I just mention it)*/
N.B. A tiny inefficiency (which can be ignored; I just mention it)
------------------------------------------------------------------
The current 'sp' may not be at head of 'global_list' -- there may be
others with same name above it.  I believe that this only legitimately
happens if 'global' declarations are in an interface body.  E.g.

	EXTERNAL s			! global_list -> s1
	...
	INTERFACE
	  FUNCTION f(s)
	    ...interface (function s)	! global_list -> s2 -> s1
	  END
	END INTERFACE
	...
	...interface (function s)	! refers to s1, which isn't first!

In this case we would have  (find_id  &&  ! found_sp), although 'sp' *is*
already in the 'global_list'.  If this happens, it results in a second
entry being installed in the 'global_list' for the same 'sp'.

However, this doesn't matter -- it doesn't affect the correctness of
anything.  It just wastes a tiny amount of storage, and is a very
rarely encountered set of circumstances anyway!
/*}}}*/

#endif
/*}}}*/
{
	/*{{{  decls*/
	
	short    scope = sp->scope;	/* scope unit of 'sp' */
	char    *name  = sp->name;	/* global name */
	bool     found_sp,		/* TRUE if 'sp' already in 'external-list' */
	         found_id;		/* TRUE if 'sp->name'    "      "     "    */
	int32    prev_attr;		/* used to give 'errmsg' */
	char    *new_str,  *prev_str;	/*  "     "    "    "    */
	Symlist *idp;
	/*}}}*/

	/*------------------------------------------------------*/
	/* Find 'name' if it's already in the 'global_list'	*/
	/*------------------------------------------------------*/
	found_sp = found_id = FALSE;

	for (idp = global_list;  idp;  idp = idp->next) {
		Symbol  *gsp = idp->sp;		/* a global symbol */
		
		if ((sp == gsp)  ||
		    ((scope != gsp->scope) && str_match (name, gsp->name)) ) {
		    	
			/*{{{  found an entry for 'name'!  Is it consistent?*/
			/*--------------------------------------------------------------*/
			/* Found 'name' in 'global_list'.  Check whether its previous	*/
			/* use as a global entity is consistent with 'global_attr'.	*/
			/*								*/
			/* N.B.:  'global_attr' can't be:				*/
			/*	AT_PROGRAM, AT_BLOCK_DATA or AT_RESULT_VAR		*/
			/* as they're only installed at the start of a new program unit,*/
			/* when 'global_list' is empty.					*/
			/*--------------------------------------------------------------*/
			
			found_id = TRUE;
			found_sp = (gsp == sp);
			
			switch (global_attr) {
			/*{{{  case  AT_FUNC :*/
			case  AT_FUNC :
			/*-----------*/
			/*{{{  EXPLANATION*/
			/*-----------------------------------------------------------------
			global_attr == AT_FUNC
			----------------------
			Called by 'process_hdr_stmt()' when an explicit interface body is
			given for a non-dummy function 'sp', which signifies that it's an
			external function.
				First, check that 'gsp' doesn't have any *inconsistent* attrs
			set;  report an error and return FALSE if it does.  It's ok if 'gsp'
			already has AT_FUNC set, as that means that it has already been declared
			(e.g. in a typedecl or interface) or used as an external function.
				If 'gsp->attrs' has AT_EXTERNAL set by itself, this means
			that 'name' has already been declared EXTERNAL but its 'proc type'
			(function or subroutine) hasn't been determined yet in this or any
			previous scope unit.  In this case, set the AT_FUNC flag in all
			existing entries for 'name' in the 'global_list'.
			[ Actually this won't happen -- see point (3) below! ]
			
			N.B.:
			-----
			(1)  A further check is performed after this loop, namely that 'sp'
			isn't the host external function (as one shouldn't give an
			interface body for the host).
			
			(2)  Note that 'gsp->attrs' may have AT_COMMON_BLOCK set together
			with AT_FUNC.  In that case, as a global entity 'gsp->name' is a
			*common block*, although it's also a local  (i.e. dummy, internal
			or module---though *not* intrinsic!) function.
				Therefore we must check for the *absence* of inconsistent
			attributes first.  Simply observing that AT_FUNC is already set
			is insufficient evidence for 'gsp' being an *external* function.
			
			(3)  When 'process_hdr_stmt()' calls this (as a result of a function
			interface body), it has already called 'install_attr (sp, AT_FUNC)'
			and the latter has successfully installed AT_FUNC in 'sp->attrs'
			if it wasn't already set.
				In detail, this means that:
			
			-- 'sp' doesn't clash with another entity type, and so is accepted as
			a function (at least in the scope unit containing the interface);
			
			-- If AT_EXTERNAL and *not* AT_DUMMY is set, then 'install_attr()'
			also sets AT_FUNC in every entry of 'sp->name' in the 'global_list'.
			(In that case everything is AOK, and nothing more need to be done!)
			
				However, it may happen that 'sp->name' is already in the
			'global_list' but *not* with flags (AT_EXTERNAL | AT_FUNC), in which
			case it has previously been declared or used as a different kind of
			external object (e.g. a common block name, subroutine, module...).
			(By the way, except in the case of a common block name, it must have
			acquired other identity in a different scope unit).
				In that case we report an error and do not install 'sp' in the
			'global_list', i.e. 'sp' is accepted as a function in the scope unit
			containing the interface body, but not as an *external* function.
			-----------------------------------------------------------------*/
			/*}}}*/
			
				if (gsp->attrs & (AT_SUBR | AT_MODULE | AT_PROGRAM | AT_BLOCK_DATA |
				                  AT_COMMON_BLOCK))
					/*------------------------------------------------------*/
					/* 'global_attr' is inconsistent with previous use.	*/
					/*------------------------------------------------------*/
					goto  error;
			
				else if (gsp->attrs & (AT_FUNC | AT_RESULT_VAR))
					/*------------------------------------------------------*/
					/* 'global_attr' is *consistent* with previous use	*/
					/* (whose 'attrs' can be EXTERNAL, FUNC, RESULT_VAR).	*/
					/* Now check that it isn't the host external function.	*/
					/*------------------------------------------------------*/
					goto  exit_loop;
			
				else {
					/*------------------------------------------------------*/
					/* gsp->attrs == AT_EXTERNAL.				*/
					/*	This means that 'name' was declared EXTERNAL in	*/
					/* a previous scope unit but its 'proc type' hasn't	*/
					/* been determined yet.  Therefore, set AT_FUNC in every*/
					/* entry for 'name' in the 'global_list'.  (N.B., the	*/
					/* 'continue' causes the entire 'global_list' to be	*/
					/* traversed).						*/
					/*							*/
					/* N.B. This won't occur -- see above 'Explanation'!	*/
					/*------------------------------------------------------*/
					gsp->attrs |= AT_FUNC;
					continue;
				}
			/*}}}*/
			/*{{{  case  AT_SUBR :*/
			case  AT_SUBR :
			/*-----------*/
			/*-----------------------------------------------------------------
			Called by 'process_hdr_stmt()' when an explicit interface body is
			given for a non-dummy subroutine 'sp', which signifies that it's
			an external subroutine.
			
			(This function is also invoked by 'process_hdr_stmt()' with
			(global_attr == AT_SUBR) when a header stmt for an external
			subroutine is processed, but in that case nothing is installed
			yet in 'global_list', so this particular code isn't executed.)
			
			The explanation is as above for AT_FUNC.
			-----------------------------------------------------------------*/
			
				if (gsp->attrs & (AT_FUNC | AT_MODULE | AT_PROGRAM | AT_BLOCK_DATA |
				                  AT_COMMON_BLOCK | AT_RESULT_VAR))
					/*------------------------------------------------------*/
					/* 'global_attr' is inconsistent with previous use.	*/
					/*------------------------------------------------------*/
					goto  error;
			
				else if (gsp->attrs & AT_SUBR)
					/*------------------------------------------------------*/
					/* 'global_attr' is *consistent* with previous use	*/
					/* (whose 'attrs' can include AT_EXTERNAL and AT_SUBR).	*/
					/* Now check that it isn't the host external subroutine.*/
					/*------------------------------------------------------*/
					goto  exit_loop;
			
				else {
					/*------------------------------------------------------*/
					/* gsp->attrs == AT_EXTERNAL.				*/
					/*	This means that 'name' was declared EXTERNAL in	*/
					/* a previous scope unit but its 'proc type' hasn't	*/
					/* been determined yet.  Therefore, set AT_SUBR in every*/
					/* entry for 'name' in the 'global_list'.  (N.B., the	*/
					/* 'continue' causes the entire 'global_list' to be	*/
					/* traversed).						*/
					/*							*/
					/* N.B. This won't occur -- see 'Explanation' in the	*/
					/* previous case!					*/
					/*------------------------------------------------------*/
					gsp->attrs |= AT_SUBR;
					continue;
				}
			/*}}}*/
			/*{{{  case  AT_MODULE :  case  AT_COMMON_BLOCK :*/
			case  AT_MODULE :
			case  AT_COMMON_BLOCK :
			/*-------------------*/
			/*{{{  EXPLANATION*/
			/*-----------------------------------------------------------------
			global_attr == AT_MODULE or AT_COMMON_BLOCK
			-------------------------------------------
			Called by 'install_attr()'.  'sp' is a MODULE referenced in a USE
			stmt, or a common block referenced in a COMMON or SAVE stmt.
				'sp' can be an object of the relevant type, 'global_attr',
			if and only if 'gsp' already has 'global_attr' set!   Otherwise,
			'name' was previously used as a different type of global entity,
			so report an error and return FALSE.
			
			N.B.:
			----
			(1)  For AT_MODULE, a further check is performed after this loop,
			namely that 'sp' isn't the host module (as one shouldn't USE the
			host module).
			
			(2)  Note that AT_COMMON_BLOCK may be set together with the attrs
			AT_FUNC, AT_SUBR and AT_RESULT_VAR, as a common block can have the
			same name as a *local* (i.e. dummy, module or internal) procedure,
			though not an external or intrinsic one.  When it is, these other flags
			signify a dummy, internal or module proc, as this function doesn't
			allow an *external* proc to have the same name as a common block
			(or any other global entity for that matter!)  See the 'Discussion'
			in case AT_FUNC.
			
			(3) Here it's important that the caller ('install_attr()') doesn't
			install AT_COMMON_BLOCK until this function has checked that it's ok.
			(Of course, it's all right if AT_COMMON_BLOCK is already installed
			in 'sp->attrs, but then there's no need to call this function anyway!)
				Failure to comply with this can lead to an error like the
			following:
			
				function f(x)	! 'f' installed in 'global_list',
						! with 'f->attrs' == AT_RESULT_VAR.
			
				  common/f/...	! If caller sets AT_COMMON_BLOCK in 'f->attrs',
						! it's accepted here, as it's already in
						! 'global_list' with AT_COMMON_BLOCK set!
						! (although it conflicts with external fn).
			-----------------------------------------------------------------*/
			/*}}}*/
			
				if (gsp->attrs & global_attr)
					/*------------------------------------------------------*/
					/* 'global_attr' is *consistent* with previous use.	*/
					/* Now, for a module, check that it isn't the host	*/
					/* module (as it's illegal for a module to USE itself).	*/
					/*------------------------------------------------------*/
					goto  exit_loop;
			
				else
					/*------------------------------------------------------*/
					/* 'global_attr' is inconsistent with previous use.	*/
					/*------------------------------------------------------*/
					goto  error;
			/*}}}*/
			/*{{{  case  AT_EXTERNAL :*/
			case  AT_EXTERNAL :
			/*---------------*/
			/*{{{  EXPLANATION*/
			/*-------------------------------------------------------------------
			global_attr == AT_EXTERNAL
			--------------------------
				Called by 'install_typedecl()' or 'install_attr()' when a
			*non-dummy* object 'sp' is declared EXTERNAL, meaning that it should
			be an external procedure or block data (though if it's the latter
			we'll never know for sure!).   The caller has checked that AT_EXTERNAL
			doesn't clash with any existing flags in 'sp->attrs', but hasn't
			installed AT_EXTERNAL yet.
				AT_EXTERNAL *isn't* installed if 'sp->name' is already in use
			for a different kind of global entity.  This includes the cases
			where previous use of 'sp->name' has (AT_FUNC | AT_SUBR) set and:
				-- 'sp' also has (AT_FUNC | AT_SUBR) set, but the *opposite*
			proc-type is set.  (In this case, by not setting AT_EXTERNAL we
			effectively keep 'sp' as a local procedure).
				-- 'sp' has a datatype / shape set (meaning it must be a
			*function*) and the previous use has AT_SUBR set.
			
			If AT_EXTERNAL *is* installed, then:
			
			if (previous use has (AT_FUNC | AT_SUBR) set)
				'sp' inherits it from the last global entry.
			
			else if (sp->attrs & (AT_FUNC | AT_SUBR))
				propagate it through the 'global_list'.
				  << N.B. The fact that 'proc-type' isn't already set in
				     'global_list' must be due to a source code error; see below. >>
			
			else if (sp->type & TY_TYPE)	<< N.B. neither 'sp' nor global_list' has
							   (AT_FUNC | AT_SUBR) set >>
				set AT_FUNC in 'sp' and all other entries for 'sp->name' in the
				global list, using  'install_global_proctype (sp, AT_FUNC)'.
				
				To be precise, the actions taken depend on the contents of
			'gsp->attrs', as follows:
			
			switch (gsp->attrs & EXTERNAL_ENTITY_TYPE) {
			~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
			AT_MODULE : AT_PROGRAM : AT_COMMON_BLOCK :
			------------------------------------------
				Inconsistent, so give an errmsg and return FALSE.
			
			AT_FUNC : AT_SUBR : (AT_FUNC | AT_EXTERNAL) : (AT_SUBR | AT_EXTERNAL) :
			-----------------------------------------------------------------------
				If 'sp->attrs' already has the *opposite* proc type installed,
			give an errmsg and return FALSE  (effectively keeping 'sp' a local
			procedure).
				Otherwise make 'sp' inherit the 'proc-type',  i.e. set
			(global_attr = (AT_EXTERNAL | proc_type)),  and exit the loop to
			install it in 'sp->attrs' (after checking that 'sp' isn't the host
			program unit).
				Therefore when 'sp' is declared EXTERNAL its proc type
			is determined from its previous use if possible.
			
			AT_EXTERNAL  &&  (sp->attrs & (AT_FUNC | AT_SUBR)) :
			----------------------------------------------------
				'sp's proc type is set but *not* 'gsp's  (even though the
			AT_EXTERNAL flag *is* set in 'gsp').  This must have arisen from
			an illegal reference to 'sp' as a procedure prior to the EXTERNAL
			statement  (e.g. as a function reference in an initialisation expr),
			in which case 'sp's proc-type was set but wasn't propagated to the
			entries for 'sp->name' in the 'global_list', since 'sp' wasn't
			known to be EXTERNAL at the time.
				Therefore we propagate 'sp's 'proc type' to every existing
			entry for 'sp->name' in the 'global_list'.
			
			default :  (AT_EXTERNAL && ! (sp->attrs & (AT_FUNC | AT_SUBR)) :
			----------------------------------------------------------------
			            AT_BLOCK_DATA : AT_RESULT_VAR) :
			            --------------------------------
				Exit the loop, where it's checked that 'sp->name' isn't
			the host program unit.  In this case 'gsp->attrs' is either:
			
			-- AT_EXTERNAL, which is OK and means that 'name' was declared EXTERNAL
			in a previous scope unit, but its proc type hasn't been determined yet.
			
			-- AT_RESULT_VAR or AT_BLOCK_DATA, which are illegal but will be
			trapped by the test after the loop (as 'gsp' is the host unit).
			We choose to trap these errors there rather than in the conventional
			'error' branch, as a more appropriate errmsg is given, e.g.:
			
				"cannot specify EXTERNAL for the host procedure %q"
			
			vs the confusing errmsg that would be given by the 'error' branch, e.g.:
			
				"%q is an external function; cannot use the same name for an
				 external procedure"
			
			N.B.  For this reason, 'install_attrs (sp, AT_EXTERNAL)' also passes
			an 'sp' whose 'sp->attrs' has AT_BLOCK_DATA set -- so the error is
			trapped here!
			-----------------------------------------------------------------*/
			/*}}}*/
			
				if (gsp->attrs & (AT_MODULE | AT_PROGRAM | AT_COMMON_BLOCK))
					/*----------------------------------------------*/
					/* EXTERNAL is inconsistent with previous use.	*/
					/*----------------------------------------------*/
					goto  error;
			
				else if (gsp->attrs & (AT_FUNC | AT_SUBR)) {
					/*------------------------------------------------------*/
					/* 'name' *is* an external proc, and its 'proc type' has*/
					/* been determined in this or another scope unit.	*/
					/*	If 'sp' has the *wrong* proc type, or it has a	*/
					/* datatype or shape set (meaning it must be a function)*/
					/* while 'sp->name' was previously used as a subroutine,*/
					/* give an errmsg and return FALSE (effectively keeping	*/
					/* it a local entity!).					*/
					/*	Otherwise install the proc type along with	*/
					/* AT_EXTERNAL in 'global_attr' so that it's inherited	*/
					/* by 'sp'.						*/
					/*							*/
					/* N.B. If (gsp == sp), then 'sp' was installed in the	*/
					/* 'global_list' when its interface body was specified.	*/
					/* The consistency of its proc type with previous usage	*/
					/* would have been checked then, so we know it's OK.	*/
					/*------------------------------------------------------*/
					int32  prev_proc_type = gsp->attrs & (AT_FUNC | AT_SUBR);
					int32  new_proc_type  =  sp->attrs & (AT_FUNC | AT_SUBR);
			
					if (new_proc_type  &&  (new_proc_type != prev_proc_type)) {
						global_attr = new_proc_type;	/* for errmsg */
						goto error;
					}
					else if ( ((sp->type & TY_TYPE) || sp->arp)  &&
					                           (prev_proc_type == AT_SUBR) ) {
						global_attr = AT_FUNC;		/* for errmsg */
						goto error;
					}
					else {
						global_attr |= prev_proc_type;	/* so 'proc type' is
						goto  exit_loop;		   installed in 'sp' */
					}
				}
				else if ((sp->attrs & (AT_FUNC | AT_SUBR)) && (gsp->attrs & AT_EXTERNAL)) {
					/*------------------------------------------------------*/
					/* 'proc type' propagated from 'sp' to the other entries*/
					/*------------------------------------------------------*/
					gsp->attrs |= (sp->attrs & (AT_FUNC | AT_SUBR));
					continue;
				}
				else	/*------------------------------------------------------*/
					/* 'gsp->attrs' is either:				*/
					/* -- AT_EXTERNAL,  which is ok and means that 'name'	*/
					/* was declared EXTERNAL in a previous scope unit	*/
					/* but its proc type hasn't been determined yet.	*/
					/* (BTW, if 'sp->type' or 'sp->arp' is defined, AT_FUNC	*/
					/* is set & propagated through 'global_list' below.	*/
					/*							*/
					/* -- AT_RESULT_VAR / AT_BLOCK_DATA,  which are illegal	*/
					/* but will be trapped by the test after the loop	*/
					/* (as 'gsp' is the host unit).				*/
					/*------------------------------------------------------*/
					goto  exit_loop;
			/*}}}*/
			}
			/*}}}*/
		}
	}
exit_loop:

	/*------------------------------------------------------*/
	/* If found, check that 'name' isn't the host unit	*/
	/*------------------------------------------------------*/
	if (found_id  &&  (global_attr != AT_COMMON_BLOCK)) {
		/*{{{  if 'name' is the program unit, give errmsg & return FALSE*/
		/*--------------------------------------------------------------*/
		/* Now, for anything but a common block, check that 'name'	*/
		/* isn't the host external unit.  If it is, the new declaration	*/
		/* is erroneous in some way, so give an errmsg and return FALSE.*/
		/*--------------------------------------------------------------*/
		Unit  *unitp = this_unitp;
			
		while ( !(unitp->attrs & EXTERNAL_LEVEL))
			unitp = unitp->prev;
			
		if (unitp->sp  &&  str_match (unitp->sp->name, name)) {
			/*------------------------------------------------------*/
			/* 'name' *is* the host program unit name!		*/
			/*							*/
			/* (N.B. We compare strings, not 'sp' pointers, since a	*/
			/* name has different symtab entries in different scope	*/
			/* units.)						*/
			/*------------------------------------------------------*/
			switch (global_attr) {
			case  AT_FUNC :
			case  AT_SUBR :
				errmsg ("must not specify an interface body for the host procedure %q",
				        name);
				return  FALSE;
				
			case  AT_MODULE :
				errmsg ("module %q must not USE itself", name);
				return  FALSE;
				
			case  AT_EXTERNAL :
			case  (AT_EXTERNAL | AT_FUNC) :
			case  (AT_EXTERNAL | AT_SUBR) :
				errmsg ("cannot specify EXTERNAL for the host %s %q",
				         ((unitp->attrs & (FUNCTION_UNIT | SUBROUTINE_UNIT)) ?
			                                    "procedure" : "program unit"),
				         name);
				return  FALSE;
			}
		}
		/*}}}*/
	}

	/*------------------------------------------------------*/
	/* AOK!  Install 'sp' if it isn't already in		*/
	/* 'global_list', set 'global_attr' & return TRUE.	*/
	/* N.B.  REALLY, WE ONLY NEED ONE ENTRY FOR EACH COMMON	*/
	/* BLOCK 'sp'!!						*/
	/*------------------------------------------------------*/
	if (! found_sp) {
		/*{{{  install new entry in 'global_list'*/
		/*--------------------------------------------------------------*/
		/* First declaration of this ID as an external name in this	*/
		/* scope unit.  Make a new record for it...			*/
		/*--------------------------------------------------------------*/
		idp = (Symlist *) ecalloc (sizeof (Symlist));
		idp->sp = sp;
		
		/*--------------------------------------------------------------*/
		/* ... and add it to the front of the 'external list'.		*/
		/*--------------------------------------------------------------*/
		idp->next = global_list;
		global_list = idp;
		/*}}}*/
	}
	sp->attrs |= global_attr;
	if ( ((sp->type & TY_TYPE) || sp->arp)  &&
	      (sp->attrs & AT_EXTERNAL)  &&  !(sp->attrs & AT_FUNC) )
		set_external_proc_type (sp, AT_FUNC);

	return TRUE;
	
error:
	/*-----------------------------*/
	/* Report error & return FALSE */
	/*-----------------------------*/
	/*{{{  'name' is a different external entity.  Give errmsg & return FALSE*/
	/*--------------------------------------------------------------*/
	/* 'name' has been used for a different kind of global entity.	*/
	/*  Give an errmsg & return FALSE.				*/
	/*--------------------------------------------------------------*/
	
	prev_attr = idp->sp->attrs & EXTERNAL_ENTITY_TYPE;	/* not 0! */
	
	if (prev_attr & AT_COMMON_BLOCK)
		prev_attr = AT_COMMON_BLOCK;	/* as could also be set with	*/
						/* FUNC, SUBR or RESULT_VAR, in	*/
						/* which case AT_COMMON_BLOCK	*/
						/* takes precedence.	*/
	else if (prev_attr & (AT_FUNC | AT_SUBR))
		prev_attr &= (AT_FUNC | AT_SUBR);
						/* to clear a possible AT_EXTERNAL */
						/* flag that may accompany it.     */
	
	/*--------------------------------------------------------------*/
	/* Now 'prev_attr' is guaranteed to be a single flag.		*/
	/*	Note that 'global_attr' can't be AT_PROGRAM,		*/
	/* AT_BLOCK_DATA or AT_RESULT_VAR (see earlier).		*/
	/*--------------------------------------------------------------*/
	switch (prev_attr) {
	case  AT_FUNC :  case  AT_RESULT_VAR :
				prev_str = str_func;		break;
	case  AT_SUBR :		prev_str = str_subr;		break;
	case  AT_MODULE :	prev_str = str_module;		break;
	case  AT_PROGRAM :	prev_str = str_program;		break;
	case  AT_BLOCK_DATA :	prev_str = str_block_data;	break;
	case  AT_COMMON_BLOCK :	prev_str = str_common_block;	break;
	case  AT_EXTERNAL :	prev_str = str_external;		break;
	}
	
	switch (global_attr) {
	case  AT_FUNC :		new_str = str_func;		break;
	case  AT_SUBR :		new_str = str_subr;		break;
	case  AT_MODULE :	new_str = str_module;		break;
	case  AT_COMMON_BLOCK :	new_str = str_common_block;	break;
	case  AT_EXTERNAL :	new_str = str_external;		break;
	}
	
	errmsg ("%q is %s; cannot use the same name for %s", name, prev_str, new_str);
	return  FALSE;
	/*}}}*/
}
/*}}}*/
/*{{{  set_external_proc_type (sp, proc_type)*/
set_external_proc_type (sp, proc_type)
/*----------------------------------*/
	Symbol  *sp;		/* the external proc */
	int32  proc_type;	/* its 'proc type' (AT_FUNC or AT_SUBR) */
/*----------------------------------------------------------------
	Called by 'install_typedecl()' and 'install_attr()'
when it needs to set AT_FUNC or AT_SUBR for a non-dummy
*external* procedure 'sp'  (i.e. one with AT_EXTERNAL set),
to set it in every symtab entry with the same name in the
'global_list'  (as they all refer to the same global entity).
	No checking is performed, since if one entry in the
'global_list' (viz. 'sp') has AT_EXTERNAL set without
(AT_FUNC | AT_SUBR), then the same applies to every other entry
with the same name.
----------------------------------------------------------------*/
{
	short    scope = sp->scope;	/* scope unit of 'sp' */
	char    *name  = sp->name;	/* global name */
	Symlist *idp;

	for (idp = global_list;  idp;  idp = idp->next) {
		Symbol  *gsp = idp->sp;		/* a global symbol */
		
		if ((sp == gsp)  ||
		    ((scope != gsp->scope) && str_match (name, gsp->name)) ) {

			gsp->attrs |= proc_type;
		}
	}
}
/*}}}*/

/*--------------------------------*/
/* Function to delete these lists */
/*--------------------------------*/
/*{{{  clear_externals () -- ok*/
clear_externals ()
/*--------------*/
/*--------------------------------------------------------------*/
/*	Called by 'close_current_unit()' at the end of each	*/
/* external program unit and subprogram, to delete:		*/
/*								*/
/* global_list :	the list of global IDs referenced in	*/
/*			the program unit;			*/
/* common_list,  namelist_list :				*/
/*			the records of common blocks and	*/
/*			namelist groups.			*/
/*								*/
/* N.B. 'clear_symtab()' has already been called, so symbol	*/
/* table entries are now non-existent!				*/
/*--------------------------------------------------------------*/
{
	Symlist  *idp,  *next_idp;
	Grouplist  *gp,  *next_gp;

	/*----------------------------*/
	/* Delete the 'global_list' */
	/*----------------------------*/
	for (idp = global_list;  idp;  idp = next_idp) {
		next_idp = idp->next;
		free ((char *) idp);
	}
	global_list = NULL;

	/*--------------------------*/
	/* Delete the 'common_list' */
	/*--------------------------*/
	for (gp = common_list;  gp;  gp = next_gp) {
		for (idp = gp->first_id;  idp;  idp = next_idp) {
			next_idp = idp->next;
			free ((char *) idp);
		}
		next_gp = gp->next;
		free ((char *) gp);
	}
	common_list = NULL;

	/*----------------------------*/
	/* Delete the 'namelist_list' */
	/*----------------------------*/
	for (gp = namelist_list;  gp;  gp = next_gp) {
		for (idp = gp->first_id;  idp;  idp = next_idp) {
			next_idp = idp->next;
			free ((char *) idp);
		}
		next_gp = gp->next;
		free ((char *) gp);
	}
	namelist_list = NULL;
}
/*}}}*/
/*}}}*/
